@@ -1095,7 +1095,7 @@ hildon_im_context_set_client_window(GtkIMContext *context,
     gdk_window_remove_filter(self->client_gdk_window,
                              (GdkFilterFunc)client_message_filter, self );
 
-    if (window == NULL)  /* Hide IM window when textwidget is unrealized */
+    if (window == NULL && self->has_focus ) /* Hide IM window when textwidget is unrealized */
     {
       if (is_internal_widget == FALSE)
       {
@@ -1431,6 +1431,9 @@ hildon_im_context_filter_keypress(GtkIMContext *context, GdkEventKey *event)
       event->keyval == GDK_KP_Enter ||
       event->keyval == GDK_ISO_Enter)
   {
+    hildon_im_context_send_key_event(self, event->type, event->state,
+                                     event->keyval, event->hardware_keycode);
+
     /* Enter advances focus as if tab was pressed */
     if (event->keyval == GDK_KP_Enter || event->keyval == GDK_ISO_Enter)
     {
@@ -1452,7 +1455,6 @@ hildon_im_context_filter_keypress(GtkIMContext *context, GdkEventKey *event)
       return FALSE;
     }
 
-    hildon_im_context_send_key_event(self, event->type, event->state, event->keyval, event->hardware_keycode);
     /* Stop both press and release events so they aren't sent to the application. */
     return TRUE;
   }
@@ -1560,7 +1562,9 @@ hildon_im_context_filter_keypress(GtkIMContext *context, GdkEventKey *event)
   /* Sticky and lock state reset */
   if (event->type == GDK_KEY_PRESS)
   {
-    if (event->keyval != GDK_Shift_L && event->keyval != GDK_Shift_R)
+    if (event->keyval != GDK_Shift_L && event->keyval != GDK_Shift_R &&
+        event->is_modifier == FALSE)
+
     {
       /* If not locked, pressing any character resets shift state */
       if ((self->mask & HILDON_IM_SHIFT_LOCK_MASK) == 0)
@@ -1568,7 +1572,8 @@ hildon_im_context_filter_keypress(GtkIMContext *context, GdkEventKey *event)
         self->mask &= ~HILDON_IM_SHIFT_STICKY_MASK;
       }
     }
-    if (event->keyval != LEVEL_KEY)
+    if (event->keyval != LEVEL_KEY &&
+        event->is_modifier == FALSE)
     {
       /* If not locked, pressing any character resets level state */
       if ((self->mask & HILDON_IM_LEVEL_LOCK_MASK) == 0)
@@ -1662,7 +1667,8 @@ hildon_im_context_filter_keypress(GtkIMContext *context, GdkEventKey *event)
                                      event->hardware_keycode);
 
     /* Non-printable characters invalidate any previous dead keys */
-    self->combining_char = 0;
+    if (event->keyval != GDK_Shift_L && event->keyval != GDK_Shift_R)
+      self->combining_char = 0;
   }
 
   if (event->keyval == GDK_BackSpace)
